<?php
namespace Datec\DatecAmino\Controller;


/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package datec_amino
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class AminoController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
	
	/**
	 * extKey
	 */
	protected $extKey;
	
	/**
	 * errors
	 */
	protected $errors;
	
	/**
	 * database
	 */
	protected $database;
	
	/**
	 * aminoRepository
	 * 
	 * @var \Datec\DatecAmino\Domain\Repository\AminoRepository
	 * @inject
	 */
	protected $aminoRepository;
	
	/**
	 * initialize current action
	 * @return void
	 */
	public function initializeAction() {
		$this->extKey = $this->request->getControllerExtensionKey();
		$this->checkConfiguration();
		
	}
	
	/**
	 * initialize searchForm action
	 *
	 * @return void
	 */
	protected function initializeSearchFormAction() {
		// allow all Properties to be mapped
		$mappingConfiguration = $this->arguments->getArgument('aminoCriteria')->getPropertyMappingConfiguration();
		$mappingConfiguration->allowAllProperties();
		$mappingConfiguration
		->setTypeConverterOption(
				'TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter',
				\TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter::CONFIGURATION_CREATION_ALLOWED,
				TRUE
		);	
	}
	
	/**
	 * initialize search action
	 *
	 * @return void
	 */
	protected function initializeSearchAction() {
		// allow all Properties to be mapped
		$mappingConfiguration = $this->arguments->getArgument('aminoCriteria')->getPropertyMappingConfiguration();
		$mappingConfiguration->allowAllProperties();
		$mappingConfiguration
		->setTypeConverterOption(
				'TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter',
				\TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter::CONFIGURATION_CREATION_ALLOWED,
				TRUE
		);
	}
	
	/**
	 * initialize searchCompounded action
	 *
	 * @return void
	 */
	protected function initializeSearchCompoundedAction() {
		// allow all Properties to be mapped
		$mappingConfiguration = $this->arguments->getArgument('aminoCriteria')->getPropertyMappingConfiguration();
		$mappingConfiguration->allowAllProperties();
		$mappingConfiguration
		->setTypeConverterOption(
				'TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter',
				\TYPO3\CMS\Extbase\Property\TypeConverter\PersistentObjectConverter::CONFIGURATION_CREATION_ALLOWED,
				TRUE
		);
	}
	
	/**
	 * action searchForm
	 * 
	 * \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 *
	 * @return void
	 */
	public function searchFormAction(\Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria = NULL) {
		if (count($this->errors) > 0 ) {
			$this->redirect('stop', 'Amino', null, array("errors" => $this->errors));
		}
		
		if ($this->feUserAccess()) {
			$operantOptions = array(
				'AND' => \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino_domain_model_aminoCriteria.operant.and',$this->extKey),
				'OR' => \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino_domain_model_aminoCriteria.operant.or',$this->extKey),
				'AND NOT' => \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino_domain_model_aminoCriteria.operant.not',$this->extKey)
			);		
		} else {			
			$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey), '', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR);
		}
		
		$this->view->assign('settings',$this->settings);
		$this->view->assign('aminoCriteria',$aminoCriteria);
		$this->view->assign('operantOptions',$operantOptions);
	}
	
	/**
	 * action search
	 * 
	 * splits to different search targets
	 * 
	 * \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 *
	 * @return void
	 */
	public function searchAction(\Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria) {
		if ($this->feUserAccess()) {
			foreach ($this->settings['tables'] as $tableModel => $targetTable) {
				if ($aminoCriteria->getTable() == $targetTable) {
					$tableModelUc = ucfirst($tableModel);
					$this->view->assign('tableModel', $tableModel);
					$this->view->assign('tableModelUc', $tableModelUc);
					
					$repositoryCallName = 'find'.$tableModelUc.'s';
					
					if (method_exists($this->aminoRepository, $repositoryCallName)) {
						$result = $this->aminoRepository->{$repositoryCallName}($aminoCriteria);						
						if ($result) {
							if(count($result[0])) {
								$this->view->assign('results',$result[0]);
								
								$this->view->assign('resultsCount',$result[1]);
								if(is_array($this->settings['ratings'])) {
									$this->view->assign('ratings',$this->settings['ratings']);
								}
							} else {
								$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.no'.$tableModelUc,$this->extKey), '', \TYPO3\CMS\Core\Messaging\FlashMessage::WARNING);
								$this->redirect('searchForm', 'Amino', 'DatecAmino', $this->request->getArguments());
							}
						} else {
							$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.dbError',$this->extKey), '', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR);
							$this->redirect('searchForm', 'Amino', 'DatecAmino', $this->request->getArguments());
						}
					} else {
						$this->addFlashMessage(\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.repositoryMethodUnknown',$this->extKey,array($repositoryCallName)), '', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR);
						$this->redirect('searchForm', 'Amino', 'DatecAmino', $this->request->getArguments());
					}
					
					$this->view->assign('settings',$this->settings);
					return;
				}
			}
			// did not forward and break before? table is not configured
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.tables_unknown',$this->extKey);
		} else {			
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey);
		}
					
		$this->redirect('stop', 'Amino', 'DatecAmino', array("errors" => $this->errors));
	}
	
	/**
	 * action searchCompounded
	 *
	 * Compounded results for listing, outpus as JSON for usage in AJAX requests
	 * request may look like this:
	 * 
	 * ==== JS Example (jQuery) ====
	   function searchAminoCompounded() {
        	// we recommend to only search, when at least one table in checkboxes is checked and a search word is given
        	if ($('input[name^="tx_datecamino_amino[aminoCriteria][tables]"]:checked').length && $('#tx_indexedsearch_sword1').val() != "") { 
            	console.log("datec_amino:searchCompounded-start");
        		var path = $(location).attr('href').split('?')[0]; 			// base path for ajax requests
	        	var pageType = '2002'; 										// pageNum for ajax requests
	        	var pageId = $('#pageId').val(); 							// pageId for ajax requests (currentPage), put a hidden field with this value on your page
	        	var data = $('input[name^="tx_datecamino_amino[aminoCriteria]"]').serialize();	// form field name convention: 'name="tx_datecamino_amino[aminoCriteria][property]"'
	        	 // example for adding indexed search searchwords to aminoCriteria (optional)
	        	data += '&tx_datecamino_amino%5BaminoCriteria%5D%5Bsword1%5D=' + $('#tx_indexedsearch_sword1').val();
	        	if ($('#tx_indexedsearch_sword2').val() != "") {
	        		 // did you know that indexed_search has a typo for the word 'operant'?
	        		var operant = 'AND';
	        		switch ($('#tx_indexedsearch_operand2').val()) {
		        		case '+' : operant = 'AND'; break;
		        		case '|' : operant = 'OR'; break;
		        		case '-' : operant = 'AND NOT'; break;
		        		default : operant = 'AND';
	        		}
	        		data += '&tx_datecamino_amino%5BaminoCriteria%5D%5Boperant%5D=' + operant;
	        		data += '&tx_datecamino_amino%5BaminoCriteria%5D%5Bsword2%5D=' + $('#tx_indexedsearch_sword2').val();
	        	}
	        	
	        	var url = path + '?id=' + pageId + '&tx_datecamino_amino[controller]=Amino&tx_datecamino_amino[action]=searchCompounded&type=' + pageType + '&' + data;
	        	console.log("datec_amino:searchCompounded-requestUrl:" + url);
	        	console.log("datec_amino:searchCompounded-searching...");
	        	$.ajax({
					async: true,
				    url: url,
				    success: function(result,status,xhr) {
				    	console.log("datec_amino:searchCompounded-search-success");
				    	console.log(result);
			        	console.log("datec_amino:searchCompounded-rendering...");
				    	var resultObj = jQuery.parseJSON(result);   // don't forget to decode the result to gain a object
				    	renderAminoResults(resultObj);              // render how ever you like	
				    		// your rendering function might do something like this: 
        					var aminoResult = jQuery.parseJSON(resultObj.sections[i].results[j]);	// iterate over sections and results, decode the single result	 
				    	$('#tx-datecamino-amino-results').show();     // container for rendered results
			        	console.log("datec_amino:searchCompounded-end");
				    },
				    error: function(error) {
				    	console.log("datec_amino:searchCompounded-search-error");
				        console.log(error);
				        var resultObj = jQuery.parseJSON(error.responseText);
				        renderAminoErrors(resultObj.errors);        // render how ever you like		 
				    	$('#tx-datecamino-amino-info').show();		// container for rendered errors        	
			        	console.log("datec_amino:searchCompounded-end");
				    }
				});
        	}
        }
	 * 
	 * 
	 * ==== LINK example ===
	  	http://www.yoursite.com?id=PAGE-ID&tx_datecamino_amino[controller]=Amino&tx_datecamino_amino[action]=searchCompounded&type=2002&tx_datecamino_amino[aminoCriteria][tables][]=rezeptur&tx_datecamino_amino[aminoCriteria][sword1]=SWORD
	 *
	 * \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 *
	 * @return string JSON encoded result object
	 */
	public function searchCompoundedAction(\Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria) {
		$resultObj = new \stdClass();
		if ($this->feUserAccess()) {
			$resultObj->aminoCriteria = $aminoCriteria->_getProperties();
			$resultObj->sections = array();
			
			// search all tables, compound result
			if (is_array($this->settings['tables'])) {
				$critTables = $aminoCriteria->getTables();
				if (count($critTables)) {
					$sectionIndex = 0;					
					foreach ($this->settings['tables'] as $tableModel => $targetTable) {
						if (in_array($targetTable, $critTables)) {
							$tableModelUc = ucfirst($tableModel);
							$repositoryCallName = 'find'.$tableModelUc.'s';
							
							if (method_exists($this->aminoRepository, $repositoryCallName)) {
								//$result = $this->aminoRepository->{$repositoryCallName}($aminoCriteria);
								if ($result) {
									if(count($result[0])) {
										header($_SERVER['SERVER_PROTOCOL']." 200 OK", true, 200);
										$resultObj->sections[$sectionIndex]->tableModel = $tableModel;
										$resultObj->sections[$sectionIndex]->tableModelUc = $tableModelUc;
										$resultObj->sections[$sectionIndex]->results = $result[0];
										$resultObj->sections[$sectionIndex]->resultsCount = $result[1];
										
									} else {
										header($_SERVER['SERVER_PROTOCOL']." 204 No Content", true, 204);
										$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.no'.$tableModelUc,$this->extKey);
									}
								} else {
									//header($_SERVER['SERVER_PROTOCOL']." 500 Internal Server Error", true, 500);
									switch ($tableModel) {
										case 'aminoPrescription' : 
											$testModel = new \Datec\DatecAmino\Domain\Model\AminoPrescription(); 
											$testModel->setId(md5(rand(10, 50)));
											$testModel->setTitle($tableModel);
											$testModel->setDosage('Etwas: Nimm etwas mehr, oder etwas weniger.');
											$testModel = $testModel->__toJson();
											break;
										case 'aminoLink' : $testModel = new \Datec\DatecAmino\Domain\Model\AminoLink(); 
											$testModel->setId(md5(rand(10, 50)));
											$testModel->setTitle($tableModel);
											$testModel = $testModel->__toJson();
											break;
										case 'aminoRequest' : $testModel = new \Datec\DatecAmino\Domain\Model\AminoRequest(); 
											$testModel->setId(md5(rand(10, 50)));
											$testModel->setTopic($tableModel);
											$testModel = $testModel->__toJson();
											break;
										default : $testModel = new NULL;
									}
									$testResult = array($testModel);
									header($_SERVER['SERVER_PROTOCOL']." 200 OK", true, 200);
									$resultObj->sections[$sectionIndex]->tableModel = $tableModel;
									$resultObj->sections[$sectionIndex]->tableModelUc = $tableModelUc;
									$resultObj->sections[$sectionIndex]->tableModelDisplayName = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino_domain_model_'.$tableModel, $this->extKey);
									$resultObj->sections[$sectionIndex]->results = $testResult;
									$resultObj->sections[$sectionIndex]->resultsCount = count($testResult);
									
									$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.dbError',$this->extKey);
								}
							} else {
								header($_SERVER['SERVER_PROTOCOL']." 500 Internal Server Error", true, 500);
								$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.repositoryMethodUnknown',$this->extKey,array($repositoryCallName));
							}
							
						}
						$sectionIndex++;
					}
					// output settings
					$resultObj->settings = $this->settings;
					// for security, do not output
					unset($resultObj->settings['aminoServerHost']);
					unset($resultObj->settings['aminoServerUser']);
					unset($resultObj->settings['aminoServerPassword']);
					unset($resultObj->settings['allowedFeUserGroups']);
					// put ratings acessible on first level
					if(is_array($this->settings['ratings'])) {
						$resultObj->ratings = $this->settings['ratings'];
					}
				} else {
					header($_SERVER['SERVER_PROTOCOL']." 204 No Content", true, 204);
					$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.noTables',$this->extKey);
				}
			} else {
				header($_SERVER['SERVER_PROTOCOL']." 500 Internal Server Error", true, 500);
				$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.tables_unknown',$this->extKey);
			}
		} else {
			header($_SERVER['SERVER_PROTOCOL']." 403 Forbidden", true, 403);
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.noFeUser', $this->extKey);
		}
				
		$resultObj->errors = $this->errors;
		
		return json_encode($resultObj);
	}
	
	/**
	 * action stop
	 *
	 * displays errors on page
	 *
	 * @return void
	 */
	public function stopAction() {
		if($this->request->hasArgument("errors")) {
			$errors = $this->request->getArgument("errors");
			foreach ($errors as $errorMsg) {
				$this->addFlashMessage($errorMsg, '', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR);
			}
		}		
	}
	
	/**
	 * if configured, the access will be granted to frontent users logged in with the allowed groups
	 * note: with this, only logged in users can use this plugin (as required by the AMINO projekt)
	 * make sure to configure 'allowedFeUserGroups' with a comma-seperated list of your pharmaceutical personel's FE-groups
	 * 
	 * @return boolean
	 */
	private function feUserAccess() {
		$access = FALSE;
		if (!isset($this->settings['allowedFeUserGroups']) || empty($this->settings['allowedFeUserGroups'])) {
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.allowedFeUserGroups',$this->extKey);
			return $access;
		}
		foreach (explode(',', $this->settings['allowedFeUserGroups']) as $allowedUsergroup) {
			$access = in_array($allowedUsergroup, explode(',', $GLOBALS['TSFE']->gr_list)); // check with allowed groups list against users access list
			if ($access) {
				break;
			}
		}
		return $access;
	}
	
	/**
	 * check plugin configuration
	 *
	 * @return void
	 */
	private function checkConfiguration() {
		if (!isset($this->settings['aminoServerHost']) || empty($this->settings['aminoServerHost'])) {
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.aminoServerHost',$this->extKey);
		}
		if (!isset($this->settings['aminoServerUser']) || empty($this->settings['aminoServerUser'])) {
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.aminoServerUser',$this->extKey);
		}
		if (!isset($this->settings['aminoServerPassword']) || empty($this->settings['aminoServerPassword'])) {
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.aminoServerPassword',$this->extKey);
		}
		
		if (!isset($this->settings['tables']) || empty($this->settings['tables'])) {
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.tables',$this->extKey);
		}
		if (!isset($this->settings['allowedFeUserGroups']) || empty($this->settings['allowedFeUserGroups'])) {
			$this->errors[] = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.technicalError',$this->extKey).\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_datecamino.errors.configuration.allowedFeUserGroups',$this->extKey);
		}
	}

}
?>