<?php
namespace Datec\DatecAmino\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012-2013 Susanne Moog, <typo3@susannemoog.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
/**
 * A repository for Amino Presciptions
 *
 */
class AminoRepository extends \TYPO3\CMS\Extbase\Persistence\Repository {

	/**
	 * @var \TYPO3\CMS\Dbal\Database\DatabaseConnection
	 * @inject
	 */
	protected $databaseConnection;

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper
	 * @inject
	 */
	protected $dataMapper;
	
	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 * @inject
	 */
	protected $configurationManager;
	
	/**
	 * settings
	 */
	protected $settings;

	/**
	 * 
	 *
	 * @return void
	 */
	public function initializeObject() {
		$this->databaseConnection = $GLOBALS['TYPO3_DB'];
		$this->settings = $this->configurationManager->getConfiguration('Settings','DatecAmino','Amino');
		$dbconf = $this->databaseConnection->conf;
		
		$this->databaseConnection->handlerCfg['aminoDBKey'] = array (
			'type' => 'native',
			'config' => array(
				'username' => $this->settings['aminoServerUser'],
				'password' => $this->settings['aminoServerPassword'],
				'host' => $this->settings['aminoServerHost'],
				'database' => 'amino',
			)
		);
		
		foreach ($this->settings[tables] as $tableModel => $table) {
			$this->databaseConnection->conf['table2handlerKeys'][$table] = 'aminoDBKey';
		}
		
		$this->databaseConnection->initialize();
		$this->databaseConnection->handler_init('aminoDBKey');
	}
	
	/**
	 * searches for land string
	 * 
	 * @param string $land
	 * @return boolean|array result array or FALSE if not found
	 */
	private function findAminoLand($land) {
		$result = FALSE;
		$table = 'land';
		$select = '*';
		$land = substr($land, 0, 1); // might be 'Y2' for bavaria, should be 'Y'
		$where = 'land = '.$this->databaseConnection->fullQuoteStr($land, $table);
		
		$res = $this->databaseConnection->exec_SELECTquery($select, $table, $where, '', '', '1');
		if ($res) {
			$result = $this->databaseConnection->sql_fetch_assoc($res);
		}
		
		return $result;
	} 

	/**
	 * finds prescriptions by amino search criteria
	 * "Rezepturen"
	 * 
	 * @param \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 * @param array $settings Plugin configuration
	 *
	 * @return array $result
	 */
	public function findAminoPrescriptions($aminoCriteria) {	
		$result = array();
		$rows = array();
		$table = $this->settings['tables']['aminoPrescription'];
		$select = '*';
		$where = 'pubgrad >= 2';
	
		// first search word is required
		$sword1 = $this->databaseConnection->quoteStr($aminoCriteria->getSword1(), $table);
		$where .= ' AND ((stichwort like "%'.$sword1.'%") ';
		$where .= ' OR (name like "%'.$sword1.'%") ';
		$where .= ' OR (synonym like "%'.$sword1.'%") ';
		$where .= ' OR (zusammensetzung like "%'.$sword1.'%") ';
		$where .= ' OR (indikation like "%'.$sword1.'%")';
		$where .= ' OR (darr like "%'.$sword1.'%"))';
	
		$sword2 = $this->databaseConnection->quoteStr($aminoCriteria->getSword2(), $table);
		if (!empty($sword2)) {
			$where .= $this->databaseConnection->quoteStr($aminoCriteria->getOperant(), $table);
			$where .= ' ((stichwort like "%'.$sword2.'%") ';
			$where .= ' OR (name like "%'.$sword2.'%") ';
			$where .= ' OR (synonym like "%'.$sword2.'%") ';
			$where .= ' OR (zusammensetzung like "%'.$sword2.'%") ';
			$where .= ' OR (indikation like "%'.$sword2.'%")';
			$where .= ' OR (darr like "%'.$sword2.'%"))';
		}
		
		$res = $this->databaseConnection->exec_SELECTquery($select, $table, $where, '', '');
		if ($res) {
			while ($row = $this->databaseConnection->sql_fetch_assoc($res)) {
				$rows[] = $row;
			}
			$result[0] = $this->dataMapper->map('\Datec\\DatecAmino\\Domain\\Model\\AminoPrescription', $rows);
			$result[1] = count($result[0]);
		} else {
			$result = $res;
		}

		return $result;
	}
	
	/**
	 * finds links by amino search criteria
	 * "Linkliste"
	 *
	 * @param \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 * @param array $settings Plugin configuration
	 *
	 * @return array $result
	 */
	public function findAminoLinks($aminoCriteria) {
		$result = array();
		$rows = array();
		$table = $this->settings['tables']['aminoLink'];
		$select = '*';
		$where = 'pubgrad >= 2';
		$orderBy = 'lfd ASC';
	
		// first search word is required
		$sword1 = $this->databaseConnection->quoteStr($aminoCriteria->getSword1(), $table);
		$where .= ' AND ((stichwort like "%'.$sword1.'%") ';
		$where .= ' OR (titel like "%'.$sword1.'%") ';
		$where .= ' OR (thema1 like "%'.$sword1.'%") ';
		$where .= ' OR (thema2 like "%'.$sword1.'%") ';
		$where .= ' OR (thema3 like "%'.$sword1.'%")';
		$where .= ' OR (anbieter like "%'.$sword1.'%")';
		$where .= ' OR (link like "%'.$sword1.'%"))';
	
		$sword2 = $this->databaseConnection->quoteStr($aminoCriteria->getSword2(), $table);
		if (!empty($sword2)) {
			$where .= $this->databaseConnection->quoteStr($aminoCriteria->getOperant(), $table);
			$where .= ' ((stichwort like "%'.$sword2.'%") ';
			$where .= ' OR (titel like "%'.$sword2.'%") ';
			$where .= ' OR (thema1 like "%'.$sword2.'%") ';
			$where .= ' OR (thema2 like "%'.$sword2.'%") ';
			$where .= ' OR (thema3 like "%'.$sword2.'%")';
			$where .= ' OR (anbieter like "%'.$sword1.'%")';
			$where .= ' OR (link like "%'.$sword2.'%"))';
		}
		
		if (in_array($table, explode(',',$this->settings['langOptions']['tables']))) {
			$where .= ' AND (';
			$langs = $aminoCriteria->getLangs();
			foreach($langs as $key => $lang) {
				if ($lang == 'other') {
					if ($key > 0) {
						$where .= ' OR';
					}
					$otherLangs = explode(',',$this->settings['langOptions']['otherLangs']);
					foreach ($otherLangs as $otherKey => $otherLang) {						
						if ($otherKey > 0) {
							$where .= ' OR';
						}
						$where .= ' sprache = "'.$otherLang.'"';
					}
				} else {
					if ($key > 0) {
						$where .= ' OR';
					}
					$where .= ' sprache = "'.$lang.'"';
				}
			}
			$where .= ')';
		}
	
		$res = $this->databaseConnection->exec_SELECTquery($select, $table, $where, '', $orderBy);
		if ($res) {
			while ($row = $this->databaseConnection->sql_fetch_assoc($res)) {
				$rows[] = $row;
			}
			$result[0] = $this->dataMapper->map('\Datec\\DatecAmino\\Domain\\Model\\AminoLink', $rows);
			$result[1] = $result[1] = count($result[0]);
		} else {
			$result = $res;
		}
	
		return $result;
	}
	
	/**
	 * finds requests by amino search criteria
	 * "Anfragen"
	 *
	 * @param \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 * @param array $settings Plugin configuration
	 *
	 * @return array $result
	 */
	public function findAminoRequests($aminoCriteria) {
		$result = array();
		$rows = array();
		$table = $this->settings['tables']['aminoRequest'];
		$select = '*';
		$where = 'pubgrad >= 2';
	
		// first search word is required
		$sword1 = $this->databaseConnection->quoteStr($aminoCriteria->getSword1(), $table);
		$where .= ' AND ((stichwort like "%'.$sword1.'%") ';
		$where .= ' OR (thema like "%'.$sword1.'%") ';
		$where .= ' OR (kommentar like "%'.$sword1.'%") ';
		$where .= ' OR (frage like "%'.$sword1.'%"))';
	
		$sword2 = $this->databaseConnection->quoteStr($aminoCriteria->getSword2(), $table);
		if (!empty($sword2)) {
			$where .= $this->databaseConnection->quoteStr($aminoCriteria->getOperant(), $table);
			$where .= ' ((stichwort like "%'.$sword2.'%") ';
			$where .= ' OR (thema like "%'.$sword2.'%") ';
			$where .= ' OR (kommentar like "%'.$sword2.'%") ';
			$where .= ' OR (frage like "%'.$sword2.'%"))';
		}
	
		$res = $this->databaseConnection->exec_SELECTquery($select, $table, $where, '', '');
		if ($res) {
			while ($row = $this->databaseConnection->sql_fetch_assoc($res)) {
				if ($this->settings['loadLandNames']) {
					$land = $this->findAminoLand($row['land']);
					if ($land) {
						$row['land'] = $land['name'];
					}
				}
				$rows[] = $row;
			}
			$result[0] = $this->dataMapper->map('\Datec\\DatecAmino\\Domain\\Model\\AminoRequest', $rows);
			$result[1] = $result[1] = count($result[0]);
		} else {
			$result = $res;
		}
	
		return $result;
	}
		
}


?>